import builtins
import inspect
import typing as tp
from collections import defaultdict

from .utils import Command


def format_key(k) -> str:
    if hasattr(k, "value"):
        return k.value.replace("c-", "Ctrl-").replace("s-", "Shift-")
    return str(k)


def _get_keybindgs():
    from prompt_toolkit.key_binding.key_bindings import Binding

    bindings: tp.List[Binding] = builtins.__xonsh__.shell.prompter.key_bindings.bindings

    keys = defaultdict(list)
    for i in bindings:

        key_formatted = " + ".join([format_key(k) for k in i.keys])
        if key_formatted.startswith("Keys"):
            key_formatted = key_formatted.replace("Keys.", "")
        keys[i.handler].append(key_formatted)

    return keys


def _grouped_by_modules():
    bindings = _get_keybindgs()
    mod_keys = defaultdict(list)
    for handle, keys in bindings.items():
        mod_keys[handle.__module__].append((handle, keys))

    handle_counter = defaultdict(int)
    single_counter = []
    for mod, handles in mod_keys.items():
        count = len(handles)
        if count > 1:
            handle_counter[mod] = len(handles)
        else:
            single_counter.append(mod)

    final_cont = defaultdict(list)
    for mod in sorted(handle_counter):
        final_cont[mod] = mod_keys[mod]

    for mod in single_counter:
        final_cont["..."].extend(mod_keys[mod])

    return final_cont


@Command
def report_key_bindings(_stdout):
    """Show current Prompt-toolkit bindings in a nice table format"""

    from rich.console import Console
    from rich.table import Table

    mod_keys = _grouped_by_modules()
    tables = []
    for mod, handles in mod_keys.items():
        mod_table = Table(
            pad_edge=False,
            # box=None,
            expand=True,
            show_lines=True,
            title=f"[green]{mod}[/green]",
        )
        mod_table.add_column("Keys", style="dim")
        mod_table.add_column("Description")
        for handle, keys in handles:
            module = f"{handle.__module__ if mod == '...' else ''}.{handle.__name__}"
            docstr = inspect.getdoc(handle) or ""
            mod_table.add_row(
                "\n".join(keys), ". ".join([docstr, f"[blue]fn: {module}[/blue]"])
            )
        tables.append(mod_table)

    console = Console()
    # https://bugs.python.org/issue37871
    # with console.pager():
    console.print(*tables)


def term_test(c):
    from rich import inspect

    inspect(c)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xontrib', 'xontrib.commands']

package_data = \
{'': ['*']}

extras_require = \
{':python_version >= "3.6" and python_version < "4.0"': ['arger>=1.2.7,<2.0.0',
                                                         'rich']}

setup_kwargs = {
    'name': 'xontrib-commands',
    'version': '0.2.2',
    'description': 'Useful xonsh-shell commands/alias functions',
    'long_description': '<p align="center">\nUseful xonsh-shell commands/alias functions\n</p>\n\n## Installation\n\nTo install use pip:\n\n``` bash\nxpip install xontrib-commands\n# or: xpip install -U git+https://github.com/jnoortheen/xontrib-commands\n```\n\n## Usage\n\n``` bash\nxontrib load commands\n\n```\n\n## building alias\n\nUse [`xontrib.commands.Command`](https://github.com/jnoortheen/xontrib-commands/blob/main/xontrib/commands.py#L9) \nto build [arger](https://github.com/jnoortheen/arger) dispatcher\nfor your functions.\n\n```py\nfrom xontrib.commands import Command\n@Command\ndef record_stats(pkg_name=".", path=".local/stats.txt"):\n    stat = $(scc @(pkg_name))\n    echo @($(date) + stat) | tee -a @(path)\n```\n\nNow a full CLI is ready\n```sh\n$ record-stats --help                                                                        \nusage: xonsh [-h] [-p PKG_NAME] [-a PATH]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -p PKG_NAME, --pkg-name PKG_NAME\n  -a PATH, --path PATH\n```\n\n## Commands\n\n- each of the commands use argparser. Please use `cmd --help` to get more info and usage examples.\n\n### 1. reload-mods\n![](./docs/2020-12-02-14-30-47.png)\n\n### 2. report-key-bindggs\n![](./docs/2020-12-02-14-30-17.png)\n\n### 3. dev\n- command to cd around fast. \n- much like https://github.com/ohmyzsh/ohmyzsh/tree/master/plugins/pj\n\n## Credits\n\nThis package was created with [xontrib cookiecutter template](https://github.com/jnoortheen/xontrib-cookiecutter).\n',
    'author': 'Noortheen Raja NJ',
    'author_email': 'jnoortheen@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jnoortheen/xontrib-commands',
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
